@@ -1,6 +1,6 @@
-import theming
+import poezio.theming
 
-class DarkTheme(theming.Theme):
+class DarkTheme(poezio.theming.Theme):
     COLOR_INFORMATION_BAR = (-1, 236)
     COLOR_STATUS_XA = (53, -1)
     COLOR_STATUS_AWAY = (214, -1)
@@ -15,7 +15,9 @@ class DarkTheme(theming.Theme):
     COLOR_MORE_INDICATOR = (6, 4)
 
     COLOR_TAB_NORMAL = (-1, 236)
+    COLOR_TAB_NONEMPTY = (-1, 236)
     COLOR_TAB_CURRENT = (-1, 16)
+    COLOR_TAB_COMPOSING = (3, 236)
     COLOR_TAB_NEW_MESSAGE = (3, 236)
     COLOR_TAB_HIGHLIGHT = (1, 236)
     COLOR_TAB_ATTENTION = (6, 236)
@@ -35,12 +37,14 @@ class DarkTheme(theming.Theme):
     COLOR_VERTICAL_TAB_NORMAL = (240, -1)
     COLOR_VERTICAL_TAB_CURRENT = (-1, 236)
     COLOR_VERTICAL_TAB_NEW_MESSAGE = (3, -1)
+    COLOR_VERTICAL_TAB_COMPOSING = (3, -1)
     COLOR_VERTICAL_TAB_HIGHLIGHT = (1, -1)
     COLOR_VERTICAL_TAB_PRIVATE = (2, -1)
     COLOR_VERTICAL_TAB_ATTENTION = (6, -1)
     COLOR_VERTICAL_TAB_DISCONNECTED = (13, -1)
 
     COLOR_INFORMATION_TEXT = (244, -1)
+    COLOR_LOG_MSG = (244, -1)
 
 
 theme = DarkTheme()
