@@ -1,5 +1,5 @@
 #!/bin/sh
-cd $(dirname "$0")
+cd $(dirname "$(readlink -f "$0")")
 if [ -z "$POEZIO_VENV" ]
 then
     POEZIO_VENV="poezio-venv"
@@ -9,7 +9,7 @@ if [ -e .git ]
 then
     args=$(git show --format='%h %ci' | head -n1)
 else
-    args="0.13.1-dev"
+    args="0.14-dev"
 fi
 
 if [ -e "$POEZIO_VENV" ]
