@@ -27,19 +27,26 @@ from poezio.plugin import BasePlugin
 
 from pygments import highlight
 from pygments.lexers import get_lexer_by_name
-from pygments.formatters import HtmlFormatter
+from pygments.formatters import HtmlFormatter  #pylint: disable=no-name-in-module
 FORMATTER = HtmlFormatter(nowrap=True, noclasses=True)
 
+
 class Plugin(BasePlugin):
     def init(self):
-        self.api.add_command('code', self.command_code,
-                usage='<language> <code>',
-                short='Sends syntax-highlighted code',
-                help='Sends syntax-highlighted code in the current tab')
+        self.api.add_command(
+            'code',
+            self.command_code,
+            usage='<language> <code>',
+            short='Sends syntax-highlighted code',
+            help='Sends syntax-highlighted code in the current tab')
 
     def command_code(self, args):
-        language, code = args.split(None, 1)
+        split = args.split(None, 1)
+        if len(split) != 2:
+            self.api.information('Usage: /code <language> <code>', 'Error')
+            return None
+        language, code = split
         lexer = get_lexer_by_name(language)
-        room = self.api.current_tab()
+        tab = self.api.current_tab()
         code = highlight(code, lexer, FORMATTER)
-        room.command_xhtml('<pre>%s</pre>' % code.rstrip('\n'))
+        tab.command_xhtml('<pre><code class="language-%s">%s</code></pre>' % (language, code.rstrip('\n')))
