@@ -1,10 +1,13 @@
 from poezio.plugin import BasePlugin
 from poezio import tabs
 
+
 class Plugin(BasePlugin):
     def init(self):
-        self.api.add_command('plugintest', self.command_plugintest, 'Test command')
-        self.api.add_tab_command(tabs.MucTab, 'plugintest', self.command_tab_plugintest, 'Test command')
+        self.api.add_command('plugintest', self.command_plugintest,
+                             'Test command')
+        self.api.add_tab_command(tabs.MucTab, 'plugintest',
+                                 self.command_tab_plugintest, 'Test command')
         self.api.add_slix_event_handler('message', self.on_message)
         self.api.information("Plugin loaded")
 
@@ -12,7 +15,8 @@ class Plugin(BasePlugin):
         self.api.information("Plugin unloaded")
 
     def on_message(self, message):
-        self.api.information("Test plugin received message: {}".format(message))
+        self.api.information(
+            "Test plugin received message: {}".format(message))
 
     def command_tab_plugintest(self, args):
         self.api.information("Command for MucTabs! With args {}".format(args))
