@@ -3,7 +3,7 @@
 # This file is part of Poezio.
 #
 # Poezio is free software: you can redistribute it and/or modify
-# it under the terms of the zlib license. See the COPYING file.
+# it under the terms of the GPL-3.0+ license. See the COPYING file.
 """
 Implements the XDG base directory specification.
 
@@ -15,11 +15,11 @@ from os import environ
 from typing import Dict
 
 # $HOME has already been checked to not be None in test_env().
-DEFAULT_PATHS = {
+DEFAULT_PATHS: Dict[str, Path] = {
     'XDG_CONFIG_HOME': Path.home() / '.config',
     'XDG_DATA_HOME': Path.home() / '.local' / 'share',
     'XDG_CACHE_HOME': Path.home() / '.cache',
-}  # type: Dict[str, Path]
+}
 
 
 def _get_directory(variable: str) -> Path:
