@@ -11,14 +11,13 @@ from poezio.user import User
 @pytest.fixture
 def user1():
     return User(
-        'nick1',
-        'member',
-        'xa',
-        'My Status!',
-        'moderator',
-        JID('foo@muc/nick1'),
-        False,
-        'red',
+        nick='nick1',
+        affiliation='member',
+        show='xa',
+        status='My Status!',
+        role='moderator',
+        jid=JID('foo@muc/nick1'),
+        color='red',
     )
 
 
@@ -41,5 +40,5 @@ def test_change_nick(user1):
 
 
 def test_change_color(user1):
-    user1.change_color('blue', deterministic=False)
+    user1.change_color('blue')
     assert user1.color == (21, -1)
