@@ -28,7 +28,7 @@ $POEZIO_PYTHON -c 'import venv' &> /dev/null || {
 }
 
 echo 'Updating poezio'
-git pull --ff-only origin master || {
+git pull --ff-only origin main || {
     echo "The script failed to update poezio."
     exit 1
 }
